%Specification tests for non-Gaussian maximum likelihood estimators,
%Fiorentini and Sentana (2021) "Quantitative economics".

% This is the main code program to replicate results in the empirical
% application of Table 5, 6 and 7.


% The Data set has been taken from 
% Angelini G., Bacchiocchi E., Caggiano G. and Fanelli L. (2018) "Uncertainty Across Volatility Regimes"


clear
clc


RECENT=0;
global NLags
global VAR_Variables_X
global VAR_Variables_Y

% General data
NLags = 4; % Number of lags of the reduced form VARs
options = optimset('LargeScale','off','MaxFunEvals',200000,'TolFun',1e-1000,'MaxIter',200000,'TolX',1e-1000);   

LimitTEST = 1.64; %
LimitTEST_Apha = 0.1;

% Graphs settings
LineWidth_IRF=2;
LineWidth_IRF_BOUNDS=1;
FontSizeIRFGraph=12;
FontSizeTimeSerieGraph=14;
HorizonIRF = 60;





% Data set
load DataSetRecent.txt


%% Data Set
DataSet = DataSetRecent(2:end,[3 7 1]);
if RECENT == 1
DataSet = DataSetRecent(331:end,[3 7 1]); %Subsample from 1988:5 324 obs
end
M=size(DataSet,2);

UM1=DataSet(:,1); % Macro Uncertainty variable
Y=DataSet(:,2); % Industrial Production variable
UMF=DataSet(:,3); % Financial Uncertainty variable


TAll = size(DataSet,1)-NLags;
M=3;
%% Reduced form estimation
% Whole sample (Not necessary for replicating Table 2)

T=TAll;
VAR_Variables_X=[ones(size(DataSet(NLags:end-1,:),1),1) DataSet(NLags:end-1,:) DataSet(NLags-1:end-2,:) DataSet(NLags-2:end-3,:) DataSet(NLags-3:end-4,:)];
VAR_Variables_Y=DataSet(NLags+1:end,:);

Beta_OLS=(VAR_Variables_X'*VAR_Variables_X)^(-1)*VAR_Variables_X'*VAR_Variables_Y;

Errors=VAR_Variables_Y-VAR_Variables_X*Beta_OLS;
Omega_LK=1/(T)*(Errors'*Errors);
SigmaOLS=1/(T)*(Errors'*Errors);
Omega_OLS=Omega_LK;
% C = cov(Errors)
PARAMOLS=[vec(Beta_OLS'); vech(SigmaOLS)];





% Student t estimator
ETA=[.2 .2 .2]';
PSI_param=zeros(51,1);
PSI_param(1:39)=vec(Beta_OLS');
dum=Omega_LK^.5;
% dum=chol(Omega_LK)';
% dum=diag(sqrt(diag(Omega_LK)));
PSI_param(40:48)=vec(dum);
PSI_param(49:51)=ETA;
 
PSI_init=PSI_param;
Savepar=PSI_param(1:48);
MaxFun=50000;
% PSI_init(1:39)=0;
[PSIML,fvalt,exitflag,grad,hessian]=SVAR_t(VAR_Variables_Y,VAR_Variables_X,PSI_init,T,MaxFun);

PSI_param=PSIML;






% Student t estimator

 
nu1=8;
nu2=8;
nu3=8;



PSI_initfixed=PSIML(1:48); 
PSI_initfixed=Savepar; 
MaxFun=50000;

[PSIMLfixed,fvaltfixed,exitflag,grad,hessian]=SVAR_tfixed(VAR_Variables_Y,VAR_Variables_X,PSI_initfixed,T,MaxFun,nu1,nu2,nu3);


%Compute Amat Bmat and Cmat at restricted ML estimators


% 


dum=PSIMLfixed(1:39,1);
Beta=reshape(dum,3,13);

dum=PSIMLfixed(40:48,1);
C=reshape(dum,3,3);
ETA=[1/nu1 1/nu2 1/nu3];
U=VAR_Variables_Y-VAR_Variables_X*Beta';
Ci=inv(C);
Eps=U*Ci';
EpsRML=Eps;
Sigma=C*C';
dum=PSIML(40:48,1);
CML=reshape(dum,3,3);
SigmaML=CML*CML';
ETAI=1./ETA;
dum=PSIML(1:39,1);
BetaML=reshape(dum,3,13);
UML=VAR_Variables_Y-VAR_Variables_X*BetaML';
CMLi=inv(CML);
EpsML=UML*CMLi';

Sigmainv=inv(Sigma);                    
  

A11=kron((VAR_Variables_X'*VAR_Variables_X),Sigmainv);


% Zst=zeros(9,6,T);
Dupn=duplicmat(3);
Knn=commatrix(3,3);

Om05inv=inv(C);
KOM=kron(Om05inv',Om05inv');
KOMKOM=KOM*(eye(9)+Knn)*KOM';
A22=0.5*Dupn'*KOMKOM*Dupn*T*0.5;

Agauss=[A11 zeros(39,6); zeros(6,39) A22];

PHIrho=zeros(3,9);
GAMrho=zeros(9,9);
ETAI=1./ETA;
GAMrho(1,1)=3*(ETAI(1)-2)/(ETAI(1)-4)-1;
GAMrho([2 4],[2 4])=1;
GAMrho([3 7],[3 7])=1;
GAMrho([6 8],[6 8])=1;
GAMrho(5,5)=3*(ETAI(2)-2)/(ETAI(2)-4)-1;
GAMrho(9,9)=3*(ETAI(3)-2)/(ETAI(3)-4)-1;



B11=A11;
B12=zeros(39,6);
B21=zeros(6,39);


B22=0.5*Dupn'*KOM*GAMrho*KOM'*Dupn*0.5*T;


Bgauss=[B11 B12; B21 B22];
Agaussi=inv(Agauss);
Cgauss=inv(Agauss)*Bgauss*inv(Agauss);









 Knn=commatrix(3,3);                   
 Mrho=zeros(15,15);

Mrho(1,1)=ETAI(1)*(ETAI(1)+1)/((ETAI(1)-2)*(ETAI(1)+3));
Mrho(2,2)=ETAI(2)*(ETAI(2)+1)/((ETAI(2)-2)*(ETAI(2)+3));
Mrho(3,3)=ETAI(3)*(ETAI(3)+1)/((ETAI(3)-2)*(ETAI(3)+3));
Mrho(4,4)=2*(ETAI(1))/(ETAI(1)+3)-1;
Mrho(8,8)=2*(ETAI(2))/(ETAI(2)+3)-1;
Mrho(12,12)=2*(ETAI(3))/(ETAI(3)+3)-1;
Mrho(5,5)=ETAI(2)*(ETAI(2)+1)/((ETAI(2)-2)*(ETAI(2)+3)); 
Mrho(6,6)=ETAI(3)*(ETAI(3)+1)/((ETAI(3)-2)*(ETAI(3)+3));
Mrho(7,7)=ETAI(1)*(ETAI(1)+1)/((ETAI(1)-2)*(ETAI(1)+3)); 
Mrho(11,11)=ETAI(2)*(ETAI(2)+1)/((ETAI(2)-2)*(ETAI(2)+3)); 
Mrho(9,9)=ETAI(3)*(ETAI(3)+1)/((ETAI(3)-2)*(ETAI(3)+3));
Mrho(10,10)=ETAI(1)*(ETAI(1)+1)/((ETAI(1)-2)*(ETAI(1)+3)); 
Mrho(4:12,4:12)=Mrho(4:12,4:12)+Knn;
Mrho(13,13)=ETAI(1)^4/4*(psi(1,ETAI(1)/2)-psi(1,(ETAI(1)+1)/2))-(ETAI(1)^4*(ETAI(1)^2+ETAI(1)-12 ))/(2*(ETAI(1)-2)^2*(ETAI(1)+1)*(ETAI(1)+3));
Mrho(14,14)=ETAI(2)^4/4*(psi(1,ETAI(2)/2)-psi(1,(ETAI(2)+1)/2))-(ETAI(2)^4*(ETAI(2)^2+ETAI(2)-12 ))/(2*(ETAI(2)-2)^2*(ETAI(2)+1)*(ETAI(2)+3));
Mrho(15,15)=ETAI(3)^4/4*(psi(1,ETAI(3)/2)-psi(1,(ETAI(3)+1)/2))-(ETAI(3)^4*(ETAI(3)^2+ETAI(3)-12 ))/(2*(ETAI(3)-2)^2*(ETAI(3)+1)*(ETAI(3)+3));
Mrho(4,13)=-6*ETAI(1)^2/((ETAI(1)-2)*(ETAI(1)+1)*(ETAI(1)+3)); 
Mrho(13,4)=Mrho(4,13); 
Mrho(8,14)=-6*ETAI(2)^2/((ETAI(2)-2)*(ETAI(2)+1)*(ETAI(2)+3));
Mrho(14,8)=Mrho(8,14); 
Mrho(12,15)=-6*ETAI(3)^2/((ETAI(3)-2)*(ETAI(3)+1)*(ETAI(3)+3));
Mrho(15,12)=Mrho(12,15); 

 

Infmat=zeros(51,51);
Infmat(49:51,49:51)=Mrho(13:15,13:15)*T;
KICi=kron(eye(3),Ci');
KICi'*Mrho(4:12,4:12)*KICi;
Infmat(40:48,40:48)=KICi*Mrho(4:12,4:12)*KICi'*T;
zzz=(kron(VAR_Variables_X,Ci));
for it=1:T,
    Infmat(1:39,1:39)=Infmat(1:39,1:39)+zzz(3*it-2:3*it,:)'*Mrho(1:3,1:3)*zzz(3*it-2:3*it,:);
end
Infmat(40:48,49:51)=KICi*Mrho(4:12,13:15)*T;
Infmat(49:51,40:48)=Infmat(40:48,49:51)';






Asyvar=inv(Infmat);
Asyvarfixed=inv(Infmat(1:48,1:48));                    
                    

%Jacobian form C to Sigma 
Ln=elimmatrix(3);
J=(eye(9)+Knn)*kron(C,eye(3));
JL=Ln*J;
Jall=[eye(39) zeros(39,12); zeros(6,39) JL zeros(6,3); zeros(3,48) eye(3)];
AsyvarS=Jall*Asyvar*Jall';
Jallfixed=[eye(39) zeros(39,9); zeros(6,39) JL];
AsyvarfixedS=Jallfixed*Asyvarfixed*Jallfixed';


DCPM=Cgauss([1:3 40:45],[1:3 40:45])-AsyvarS([1:3 40:45],[1:3 40:45]); 
DCPM1=Cgauss([1:3],[1:3])-AsyvarS([1:3],[1:3]); 
DCPM2=Cgauss([40:45],[40:45])-AsyvarS([40:45],[40:45]);


HAUSA1M=(PARAMOLS([1:3 40:45])'-[PSIML(1:3)' vech(SigmaML)'])* ...
          inv(DCPM)*(PARAMOLS([1:3 40:45])'-[PSIML(1:3)' vech(SigmaML)'])';   
HAUSA1M1=(PARAMOLS([1:3])'-[PSIML(1:3)'])* ...
          inv(DCPM1)*(PARAMOLS([1:3])'-[PSIML(1:3)'])';  
HAUSA1M2=(PARAMOLS([40:45])'-[vech(SigmaML)'])* ...
          inv(DCPM2)*(PARAMOLS([40:45])'-[vech(SigmaML)'])';   


DD=DCPM; %used in DW3



DCPM2=Asyvar([40 44 48],[40 44 48])-Asyvarfixed([40 44 48],[40 44 48]);

FHAUSA1M2=(PSIML([40 44 48])'-PSIMLfixed([40 44 48])')* ...
          inv(DCPM2)*(PSIML([40 44 48])-PSIMLfixed([40 44 48])); 
FHAUSA1M2C1=(PSIML([40])'-PSIMLfixed([40])')* ...
          inv(DCPM2(1,1))*(PSIML([40])-PSIMLfixed([40])); 
FHAUSA1M2C2=(PSIML([44])'-PSIMLfixed([44])')* ...
          inv(DCPM2(2,2))*(PSIML([44])-PSIMLfixed([44])); 
FHAUSA1M2C3=(PSIML([48])'-PSIMLfixed([48])')* ...
          inv(DCPM2(3,3))*(PSIML([48])-PSIMLfixed([48])); 
     
Feta=(PSIML([49:51])'-ETA)* ...
          inv(Asyvar(49:51,49:51))*(PSIML([49:51])'-ETA)';   
Feta1=(PSIML([49])'-ETA(1))* ...
          inv(Asyvar(49,49))*(PSIML([49])'-ETA(1))';   
Feta2=(PSIML([50])'-ETA(2))* ...
          inv(Asyvar(50,50))*(PSIML([50])'-ETA(2))';      
Feta3=(PSIML([51])'-ETA(3))* ...
          inv(Asyvar(51,51))*(PSIML([51])'-ETA(3))';      
                    
LRtest=2*(fvaltfixed-fvalt);                   
                    



        %Compute Gaussian score at Student estimate
         PMLscore=zeros(45,1);
         zz=kron(VAR_Variables_X,eye(3));
        Errors=VAR_Variables_Y-VAR_Variables_X*BetaML';
        iomega=inv(SigmaML);
        kronoo=kron(iomega,iomega);
        for it=1:T
         PMLscore(1:39)=PMLscore(1:39)+ zz(3*it-2:3*it,:)'*iomega*Errors(it,:)';
%          PMLscore(7:9)=PMLscore(7:9)+0.5*Dupn'*kronoo*vec(Errors(it,:)'*Errors(it,:)-SigmaML);
        end
        PMLscore(40:45)=0.5*Dupn'*kronoo*vec(Errors'*Errors-T*SigmaML);  
        
        
        
%       orthogonalize score(drif and vech(omega)) wrt PHI
         regrcoeff=Agauss([1:3 40:45],4:39)*inv(Agauss(4:39,4:39));
         stmall=PMLscore([1:3 40:45])-regrcoeff*PMLscore(4:39);
         DLAM=inv(Agaussi([1:3 40:45],[1:3 40:45]));
         DW3M=stmall'*inv(DLAM*DD*DLAM)*stmall;
         DLAM1=inv(Agaussi(1:3,1:3));
         DW3M1=stmall(1:3)'*inv(DLAM1*DD(1:3,1:3)*DLAM1)*stmall(1:3);
         DLAM2=inv(Agaussi(40:45,40:45));
         DW3M2=stmall(4:9)'*inv(DLAM2*DD(4:9,4:9)*DLAM2)*stmall(4:9);
         
       
          
       
         
%Compute Student score at Gaussian estimate (Rotation parameters and etas evaluated as A B C and Infmat)
U=VAR_Variables_Y-VAR_Variables_X*Beta_OLS;
%Determine rotation
SigmaLR=chol(Sigma)'; %Find Cholesky of Sigma_RML
Qx=inv(SigmaLR)*C;    %Find RML Rotation
% Control 
if sum(sum((SigmaLR*Qx-C).^2)) > 0.00000000000000000000001
    disp('Maremma maiala')
    stop
end
if abs(det(Qx)-1) > 0.00000000001
    disp('Maremma budella')
%     ivolta=ivolta+1;
%     detneg(ivolta)=irep;
%     stop
end

SigmaL=chol(Omega_OLS)';%Find gaussian Cholesky
Cibrid=SigmaL*Qx;      %Find ibrid gaussian C


%Evaluate the score at Beta_OLS Cibrid ETAfixed
Ci=inv(Cibrid);
Eps=U*Ci';

deltat=zeros(T,3);

for it=1:T,
    for i=1:3
        deltat(it,i)=(ETA(i)+1)/(1*(1-2*ETA(i))+ETA(i)*Eps(it,i)^2);
    end
end
% 
for i=1:3
dcdeta(i)=1/(2*ETA(i)*(1-2*ETA(i)))-(psi((ETA(i)+1)/(2*ETA(i)))-psi(1/(2*ETA(i))))/(2*ETA(i)^2);
end
% 
deta=zeros(T,3);
for it=1:T,
    for i=1:3
        deta(it,i)=-deltat(it,i)*Eps(it,i)^2/(2*ETA(i)*(1-2*ETA(i)))+log(1+Eps(it,i)^2*(ETA(i)/(1-2*ETA(i))))/(2*ETA(i)^2);
    end
end

        


zzz=(kron(VAR_Variables_X,Ci));
MLscore=zeros(51,1);
for it=1:T
    MLscore(1:39)=MLscore(1:39)+zzz(3*it-2:3*it,:)'*...
        [deltat(it,1)*Eps(it,1);deltat(it,2)*Eps(it,2);;deltat(it,3)*Eps(it,3)];
    MLscore(49:51)=MLscore(49:51)+([dcdeta+deta(it,1:3)])';
end
MLscore(40:48)=kron(eye(3),Ci')*(-vec(T*eye(3)-(deltat.*Eps)'*Eps));


% Smatrix=[0 -1 1 0]';
Smatrix=[0 1 0 -1 0 0 0 0 0 ; 0 0 1 0 0 0 -1 0 0; 0 0 0 0 0 1 0 -1 0]';



%Find Jacobian from Cibrid to SigmaL and Qx and from SigmaL to Sigma 
%Jacobian evaluated at RML
Hx=(eye(3)-Qx)*inv(eye(3)+Qx);
dvecQdx=-kron(inv(eye(3)-Hx),eye(3))*(eye(9)+kron(eye(3),Qx))*Smatrix;
% prova=-kron(inv(eye(3)-Hx),eye(3))*S-kron(inv(eye(3)+Hx'),(eye(3)-Hx)*inv(eye(3)+Hx))*S;
dvecCdx=kron(eye(3),SigmaLR)*dvecQdx; %Checked with DELTAS %Notice I am using Cholesky of Sigma RML
dvecCdvechSigmaL=kron(Qx',eye(3))*Ln'; %Checked with DELTAS %This matrix has six elements equal to zero!

%Compute dvechSigmaLdvechSigma
Uglymat15=Ln*kron(SigmaLR,eye(3))*Ln'+Ln*kron(eye(3),SigmaLR)*Knn*Ln';
dvechSigmaLdvechSigma=inv(Uglymat15); %checked with DELTAS
%Compute dvechSigmaLdvechSigma
dvecCdvechSigma=dvecCdvechSigmaL*dvechSigmaLdvechSigma; %Checked with DELTAS%this matrix has four elements equal to zero!
%Obtain score wrt vech(Sigma) and x
MLscorerep=MLscore;
MLscorerep(40:45)=dvecCdvechSigma'*MLscore(40:48); %Notice Jacobian is transposed
MLscorerep(46:48)=dvecCdx'*MLscore(40:48);          %Notice Jacobian is transposed
%Orthogonalize the score wrt Drifts and vech(Sigma) wrt PHI, x and eta
%But first need to obtain Infmatrep 

JACOB=eye(51);
% JACOB(7:10,7:9)=dvecCdvechSigma;
% JACOB(7:10,10)=dvecCdx;
JACOB(40:48,40:45)=dvecCdvechSigma;
JACOB(40:48,46:48)=dvecCdx;
Infmatrep=JACOB'*Infmat*JACOB;
% regrcoeff=Infmatrep([1:2 7:9],[3:6 10:12])*inv(Infmatrep([3:6 10:12],[3:6 10:12]));
% MLscoretest=MLscorerep([1:2 7:9])-regrcoeff*MLscorerep([3:6 10:12]);
regrcoeff=Infmatrep([1:3 40:45],[4:39 46:51])*inv(Infmatrep([4:39 46:51],[4:39 46:51]));
MLscoretest=MLscorerep([1:3 40:45])-regrcoeff*MLscorerep([4:39 46:51]);
% Now we can compute the DW2M tests 
Asyvarrep=inv(Infmatrep); %Notice this is block diagonal between drifts and vech(Sigma)
DUMinv=inv(Asyvarrep([1:3 40:45],[1:3 40:45]));
DD2=DUMinv*Cgauss([1:3 40:45],[1:3 40:45])*DUMinv-DUMinv;
 
DW2M=MLscoretest'*inv(DD2)*MLscoretest;
DW2M1=MLscoretest(1:3)'*inv(DD2(1:3,1:3))*MLscoretest(1:3);
DW2M2=MLscoretest(4:9)'*inv(DD2(4:9,4:9))*MLscoretest(4:9);


 
       %Compute restricted student score at student estimate
      

U=VAR_Variables_Y-VAR_Variables_X*BetaML';
Ci=inv(CML);
Eps=U*Ci';
 
 
deltat=zeros(T,3);

for it=1:T,
    for i=1:3
        deltat(it,i)=(ETA(i)+1)/(1*(1-2*ETA(i))+ETA(i)*Eps(it,i)^2);
    end
end 


zzz=(kron(VAR_Variables_X,Ci));
MLfixedscore=zeros(48,1);
for it=1:T
    MLfixedscore(1:39)=MLfixedscore(1:39)+zzz(3*it-2:3*it,:)'* ...
        [deltat(it,1)*Eps(it,1);deltat(it,2)*Eps(it,2);deltat(it,3)*Eps(it,3)];
 end
MLfixedscore(40:48)=kron(eye(3),Ci')*(-vec(T*eye(3)-(deltat.*Eps)'*Eps));


%Orthogonalize the score wrt drift PHI and C12 C21 C13 C31 C23 C32

% regrcoeff=Infmat([7 10],[1:6 8:9])*inv(Infmat([1:6 8:9],[1:6 8:9]));
% MLscoretest=MLfixedscore([7 10])-regrcoeff*MLfixedscore([1:6 8:9]);
regrcoeff=Infmat([40 44 48],[1:39 41:43 45:47])*inv(Infmat([1:39 41:43 45:47],[1:39 41:43 45:47]));
MLscoretest=MLfixedscore([40 44 48])-regrcoeff*MLfixedscore([1:39 41:43 45:47]);

DUMinv=inv(Asyvarfixed([40 44 48],[40 44 48]));
DD3=DUMinv*Asyvar([40 44 48],[40 44 48])*DUMinv-DUMinv;
FW2M2=MLscoretest'*inv(DD3)*MLscoretest;
 

 
%Compute student score at restricted student estimate

U=VAR_Variables_Y-VAR_Variables_X*Beta';
Ci=inv(C);
Eps=U*Ci';
 
 

deltat=zeros(T,3);

for it=1:T,
    for i=1:3
        deltat(it,i)=(ETA(i)+1)/(1*(1-2*ETA(i))+ETA(i)*Eps(it,i)^2);
    end
end
% 
for i=1:3
dcdeta(i)=1/(2*ETA(i)*(1-2*ETA(i)))-(psi((ETA(i)+1)/(2*ETA(i)))-psi(1/(2*ETA(i))))/(2*ETA(i)^2);
end
% 
deta=zeros(T,3);
for it=1:T,
    for i=1:3
        deta(it,i)=-deltat(it,i)*Eps(it,i)^2/(2*ETA(i)*(1-2*ETA(i)))+log(1+Eps(it,i)^2*(ETA(i)/(1-2*ETA(i))))/(2*ETA(i)^2);
    end
end 
 



zzz=(kron(VAR_Variables_X,Ci));
MLscoref=zeros(51,1);
for it=1:T
    MLscoref(1:39)=MLscoref(1:39)+zzz(3*it-2:3*it,:)'* ...
        [deltat(it,1)*Eps(it,1);deltat(it,2)*Eps(it,2);deltat(it,3)*Eps(it,3)];
    MLscoref(49:51)=MLscoref(49:51)+([dcdeta+deta(it,1:3)])';
end
MLscoref(40:48)=kron(eye(3),Ci')*(-vec(T*eye(3)-(deltat.*Eps)'*Eps));



%Orthogonalize the score wrt drift PHI C12 C21 C13 C31 C23 C32 and ETA

regrcoeff=Infmat([40 44 48],[1:39 41:43 45:47 49:51])*inv(Infmat([1:39 41:43 45:47 49:51],[1:39 41:43 45:47 49:51]));
MLscoretest=MLscoref([40 44 48])-regrcoeff*MLscoref([1:39 41:43 45:47 49:51]);

DUMinv=inv(Asyvar([40 44 48],[40 44 48]));
DD3=DUMinv-DUMinv*Asyvarfixed([40 44 48],[40 44 48])*DUMinv;

FW3M2=MLscoretest'*inv(DD3)*MLscoretest;
ALM=MLscoref(49:51)'*Asyvar([49:51],[49:51])*MLscoref(49:51);




 